@@ -3,18 +3,21 @@
 
 '''A FlickrAPI interface.
 
+The main functionality can be found in the `flickrapi.FlickrAPI`
+class.
+
 See `the FlickrAPI homepage`_ for more info.
 
-.. _`the FlickrAPI homepage`: http://flickrapi.sf.net/
+.. _`the FlickrAPI homepage`: http://stuvel.eu/projects/flickrapi
 '''
 
-__version__ = '1.1'
+__version__ = '1.3'
 __all__ = ('FlickrAPI', 'IllegalArgumentException', 'FlickrError',
-        'XMLNode', 'set_log_level', '__version__')
+        'CancelUpload', 'XMLNode', 'set_log_level', '__version__')
 __author__ = u'Sybren St\u00fcvel'.encode('utf-8')
 
 # Copyright (c) 2007 by the respective coders, see
-# http://flickrapi.sf.net/
+# http://www.stuvel.eu/projects/flickrapi
 #
 # This code is subject to the Python licence, as can be read on
 # http://www.python.org/download/releases/2.5.2/license/
@@ -42,7 +45,6 @@ __author__ = u'Sybren St\u00fcvel'.encode('utf-8')
 # SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 
 import sys
-import md5
 import urllib
 import urllib2
 import mimetools
@@ -51,10 +53,15 @@ import logging
 import copy
 import webbrowser
 
-from flickrapi.tokencache import TokenCache, SimpleTokenCache
+# Smartly import hashlib and fall back on md5
+try: from hashlib import md5
+except ImportError: from md5 import md5
+
+from flickrapi.tokencache import TokenCache, SimpleTokenCache, \
+        LockingTokenCache
 from flickrapi.xmlnode import XMLNode
 from flickrapi.multipart import Part, Multipart, FilePart
-from flickrapi.exceptions import IllegalArgumentException, FlickrError
+from flickrapi.exceptions import *
 from flickrapi.cache import SimpleCache
 from flickrapi import reportinghttp
 
@@ -97,13 +104,13 @@ def debug(method):
 
     return debugged
 
+
 # REST parsers, {format: parser_method, ...}. Fill by using the
 # @rest_parser(format) function decorator
 rest_parsers = {}
-
 def rest_parser(format):
-    '''Function decorator, use this to mark a function as the parser for REST as
-    returned by Flickr.
+    '''Method decorator, use this to mark a function as the parser for
+    REST as returned by Flickr.
     '''
 
     def decorate_parser(method):
@@ -112,7 +119,27 @@ def rest_parser(format):
 
     return decorate_parser
 
-class FlickrAPI:
+def require_format(required_format):
+    '''Method decorator, raises a ValueError when the decorated method
+    is called if the default format is not set to ``required_format``.
+    '''
+
+    def decorator(method):
+        def decorated(self, *args, **kwargs):
+            # If everything is okay, call the method
+            if self.default_format == required_format:
+                return method(self, *args, **kwargs)
+
+            # Otherwise raise an exception
+            msg = 'Function %s requires that you use ' \
+                  'ElementTree ("etree") as the communication format, ' \
+                  'while the current format is set to "%s".'
+            raise ValueError(msg % (method.func_name, self.default_format))
+
+        return decorated
+    return decorator
+
+class FlickrAPI(object):
     """Encapsulates Flickr functionality.
     
     Example usage::
@@ -128,8 +155,9 @@ class FlickrAPI:
     flickr_upload_form = "/services/upload/"
     flickr_replace_form = "/services/replace/"
 
-    def __init__(self, api_key, secret=None, fail_on_error=None, username=None,
-            token=None, format='xmlnode', store_token=True, cache=False):
+    def __init__(self, api_key, secret=None, username=None,
+            token=None, format='etree', store_token=True,
+            cache=False):
         """Construct a new FlickrAPI instance for a given API key
         and secret.
         
@@ -139,11 +167,6 @@ class FlickrAPI:
         secret
             The secret belonging to the API key.
         
-        fail_on_error
-            If False, errors won't be checked by the FlickrAPI module.
-            Deprecated, don't use this parameter, just handle the FlickrError
-            exceptions.
-        
         username
             Used to identify the appropriate authentication token for a
             certain user.
@@ -172,15 +195,8 @@ class FlickrAPI:
             >>> f.cache = SimpleCache(timeout=5, max_entries=100)
         """
         
-        if fail_on_error is not None:
-            LOG.warn("fail_on_error has been deprecated. Remove this "
-                     "parameter and just handle the FlickrError exceptions.")
-        else:
-            fail_on_error = True
-
         self.api_key = api_key
         self.secret = secret
-        self.fail_on_error = fail_on_error
         self.default_format = format
         
         self.__handler_cache = {}
@@ -213,7 +229,10 @@ class FlickrAPI:
         API. Used for tab completion in IPython.
         '''
 
-        rsp = self.reflection_getMethods(format='etree')
+        try:
+            rsp = self.reflection_getMethods(format='etree')
+        except FlickrError:
+            return None
 
         def tr(name):
             '''Translates Flickr names to something that can be called
@@ -232,7 +251,7 @@ class FlickrAPI:
         '''Parses a REST XML response from Flickr into an XMLNode object.'''
 
         rsp = XMLNode.parse(rest_xml, store_xml=True)
-        if rsp['stat'] == 'ok' or not self.fail_on_error:
+        if rsp['stat'] == 'ok':
             return rsp
         
         err = rsp.err[0]
@@ -242,16 +261,22 @@ class FlickrAPI:
     def parse_etree(self, rest_xml):
         '''Parses a REST XML response from Flickr into an ElementTree object.'''
 
-        # Only import it here, to maintain Python 2.4 compatibility
-        import xml.etree.ElementTree
+        try:
+            import xml.etree.ElementTree as ElementTree
+        except ImportError:
+            # For Python 2.4 compatibility:
+            try:
+                import elementtree.ElementTree as ElementTree
+            except ImportError:
+                raise ImportError("You need to install "
+                    "ElementTree for using the etree format")
 
-        rsp = xml.etree.ElementTree.fromstring(rest_xml)
-        if rsp.attrib['stat'] == 'ok' or not self.fail_on_error:
+        rsp = ElementTree.fromstring(rest_xml)
+        if rsp.attrib['stat'] == 'ok':
             return rsp
         
         err = rsp.find('err')
-        raise FlickrError(u'Error: %s: %s' % (
-            err.attrib['code'], err.attrib['msg']))
+        raise FlickrError(u'Error: %(code)s: %(msg)s' % err.attrib)
 
     def sign(self, dictionary):
         """Calculate the flickr signature for a set of params.
@@ -272,9 +297,7 @@ class FlickrAPI:
                         "argument %s (%r) should have been UTF-8 by now"
                         % (key, datum))
             data.append(datum)
-        
-        md5_hash = md5.new()
-        md5_hash.update(''.join(data))
+        md5_hash = md5(''.join(data))
         return md5_hash.hexdigest()
 
     def encode_and_sign(self, dictionary):
@@ -293,10 +316,10 @@ class FlickrAPI:
         Example::
 
             flickr.auth_getFrob(api_key="AAAAAA")
-            xmlnode = flickr.photos_getInfo(photo_id='1234')
+            etree = flickr.photos_getInfo(photo_id='1234')
+            etree = flickr.photos_getInfo(photo_id='1234', format='etree')
             xmlnode = flickr.photos_getInfo(photo_id='1234', format='xmlnode')
             json = flickr.photos_getInfo(photo_id='1234', format='json')
-            etree = flickr.photos_getInfo(photo_id='1234', format='etree')
         """
 
         # Refuse to act as a proxy for unimplemented special methods
@@ -447,7 +470,29 @@ class FlickrAPI:
         return "http://%s%s?%s" % (FlickrAPI.flickr_host, \
             FlickrAPI.flickr_auth_form, encoded)
 
-    def upload(self, filename, callback=None, **arg):
+    def __extract_upload_response_format(self, kwargs):
+        '''Returns the response format given in kwargs['format'], or
+        the default format if there is no such key.
+
+        If kwargs contains 'format', it is removed from kwargs.
+
+        If the format isn't compatible with Flickr's upload response
+        type, a FlickrError exception is raised.
+        '''
+
+        # Figure out the response format
+        format = kwargs.get('format', self.default_format)
+        if format not in rest_parsers and format != 'rest':
+            raise FlickrError('Format %s not supported for uploading '
+                              'photos' % format)
+
+        # The format shouldn't be used in the request to Flickr.
+        if 'format' in kwargs:
+            del kwargs['format']
+
+        return format
+
+    def upload(self, filename, callback=None, **kwargs):
         """Upload a file to flickr.
 
         Be extra careful you spell the parameters correctly, or you will
@@ -474,58 +519,26 @@ class FlickrAPI:
         is_family
             "1" or "0" whether family can see the photo while it's
             marked as private
+        content_type
+            Set to "1" for Photo, "2" for Screenshot, or "3" for Other.
+        hidden
+            Set to "1" to keep the photo in global search results, "2"
+            to hide from public searches.
+        format
+            The response format. You can only choose between the
+            parsed responses or 'rest' for plain REST.
 
         The callback method should take two parameters:
-        def callback(progress, done)
+        ``def callback(progress, done)``
         
         Progress is a number between 0 and 100, and done is a boolean
         that's true only when the upload is done.
-        
-        For now, the callback gets a 'done' twice, once for the HTTP
-        headers, once for the body.
         """
 
-        if not filename:
-            raise IllegalArgumentException("filename must be specified")
-        
-        # verify key names
-        required_params = ('api_key', 'auth_token', 'api_sig')
-        optional_params = ('title', 'description', 'tags', 'is_public', 
-                           'is_friend', 'is_family')
-        possible_args = required_params + optional_params
-        
-        for a in arg.keys():
-            if a not in possible_args:
-                raise IllegalArgumentException("Unknown parameter "
-                        "'%s' sent to FlickrAPI.upload" % a)
-
-        arguments = {'auth_token': self.token_cache.token,
-                     'api_key': self.api_key}
-        arguments.update(arg)
-
-        # Convert to UTF-8 if an argument is an Unicode string
-        arg = make_utf8(arguments)
-        
-        if self.secret:
-            arg["api_sig"] = self.sign(arg)
-        url = "http://" + FlickrAPI.flickr_host + FlickrAPI.flickr_upload_form
-
-        # construct POST data
-        body = Multipart()
-
-        for a in required_params + optional_params:
-            if a not in arg:
-                continue
-            
-            part = Part({'name': a}, arg[a])
-            body.attach(part)
-
-        filepart = FilePart({'name': 'photo'}, filename, 'image/jpeg')
-        body.attach(filepart)
-
-        return self.__send_multipart(url, body, callback)
+        return self.__upload_to_form(FlickrAPI.flickr_upload_form,
+                filename, callback, **kwargs)
     
-    def replace(self, filename, photo_id):
+    def replace(self, filename, photo_id, callback=None, **kwargs):
         """Replace an existing photo.
 
         Supported parameters:
@@ -534,44 +547,69 @@ class FlickrAPI:
             name of a file to upload
         photo_id
             the ID of the photo to replace
+        callback
+            method that gets progress reports
+        format
+            The response format. You can only choose between the
+            parsed responses or 'rest' for plain REST. Defaults to the
+            format passed to the constructor.
+
+        The callback parameter has the same semantics as described in the
+        ``upload`` function.
         """
         
-        if not filename:
-            raise IllegalArgumentException("filename must be specified")
         if not photo_id:
             raise IllegalArgumentException("photo_id must be specified")
 
-        args = {'filename': filename,
-                'photo_id': photo_id,
-                'auth_token': self.token_cache.token,
-                'api_key': self.api_key}
+        kwargs['photo_id'] = photo_id
+        return self.__upload_to_form( FlickrAPI.flickr_replace_form,
+                filename, callback, **kwargs)
+        
+    def __upload_to_form(self, form_url, filename, callback, **kwargs):
+        '''Uploads a photo - can be used to either upload a new photo
+        or replace an existing one.
 
-        args = make_utf8(args)
+        form_url must be either ``FlickrAPI.flickr_replace_form`` or
+        ``FlickrAPI.flickr_upload_form``.
+        '''
+
+        if not filename:
+            raise IllegalArgumentException("filename must be specified")
+        if not self.token_cache.token:
+            raise IllegalArgumentException("Authentication is required")
+
+        # Figure out the response format
+        format = self.__extract_upload_response_format(kwargs)
+
+        # Update the arguments with the ones the user won't have to supply
+        arguments = {'auth_token': self.token_cache.token,
+                     'api_key': self.api_key}
+        arguments.update(kwargs)
+
+        # Convert to UTF-8 if an argument is an Unicode string
+        kwargs = make_utf8(arguments)
         
         if self.secret:
-            args["api_sig"] = self.sign(args)
-        url = "http://" + FlickrAPI.flickr_host + FlickrAPI.flickr_replace_form
+            kwargs["api_sig"] = self.sign(kwargs)
+        url = "http://%s%s" % (FlickrAPI.flickr_host, form_url)
 
         # construct POST data
         body = Multipart()
 
-        for arg, value in args.iteritems():
-            # No part for the filename
-            if value == 'filename':
-                continue
-            
+        for arg, value in kwargs.iteritems():
             part = Part({'name': arg}, value)
             body.attach(part)
 
         filepart = FilePart({'name': 'photo'}, filename, 'image/jpeg')
         body.attach(filepart)
 
-        return self.__send_multipart(url, body)
+        return self.__wrap_in_parser(self.__send_multipart, format,
+                url, body, callback)
 
     def __send_multipart(self, url, body, progress_callback=None):
         '''Sends a Multipart object to an URL.
         
-        Returns the resulting XML from Flickr.
+        Returns the resulting unparsed XML from Flickr.
         '''
 
         LOG.debug("Uploading to %s" % url)
@@ -581,67 +619,26 @@ class FlickrAPI:
         (header, value) = body.header()
         request.add_header(header, value)
         
-        if progress_callback:
-            response = reportinghttp.urlopen(request, progress_callback)
-        else:
+        if not progress_callback:
+            # Just use urllib2 if there is no progress callback
+            # function
             response = urllib2.urlopen(request)
-        rspXML = response.read()
+            return response.read()
 
-        return self.parse_xmlnode(rspXML)
+        def __upload_callback(percentage, done, seen_header=[False]):
+            '''Filters out the progress report on the HTTP header'''
 
-    @classmethod
-    def test_failure(cls, rsp, exception_on_error=True):
-        """Exit app if the rsp XMLNode indicates failure."""
+            # Call the user's progress callback when we've filtered
+            # out the HTTP header
+            if seen_header[0]:
+                return progress_callback(percentage, done)            
+            
+            # Remember the first time we hit 'done'.
+            if done:
+                seen_header[0] = True
 
-        LOG.warn("FlickrAPI.test_failure has been deprecated and will be "
-                 "removed in FlickrAPI version 1.2.")
-
-        if rsp['stat'] != "fail":
-            return
-        
-        message = cls.get_printable_error(rsp)
-        LOG.error(message)
-        
-        if exception_on_error:
-            raise FlickrError(message)
-
-    @classmethod
-    def get_printable_error(cls, rsp):
-        """Return a printed error message string of an XMLNode Flickr response."""
-
-        LOG.warn("FlickrAPI.get_printable_error has been deprecated "
-                 "and will be removed in FlickrAPI version 1.2.")
-
-        return "%s: error %s: %s" % (rsp.name, \
-            cls.get_rsp_error_code(rsp), cls.get_rsp_error_msg(rsp))
-
-    @classmethod
-    def get_rsp_error_code(cls, rsp):
-        """Return the error code of an XMLNode Flickr response, or 0 if no
-        error.
-        """
-
-        LOG.warn("FlickrAPI.get_rsp_error_code has been deprecated and will be "
-                 "removed in FlickrAPI version 1.2.")
-
-        if rsp['stat'] == "fail":
-            return int(rsp.err[0]['code'])
-
-        return 0
-
-    @classmethod
-    def get_rsp_error_msg(cls, rsp):
-        """Return the error message of an XMLNode Flickr response, or "Success"
-        if no error.
-        """
-
-        LOG.warn("FlickrAPI.get_rsp_error_msg has been deprecated and will be "
-                 "removed in FlickrAPI version 1.2.")
-
-        if rsp['stat'] == "fail":
-            return rsp.err[0]['msg']
-
-        return "Success"
+        response = reportinghttp.urlopen(request, __upload_callback)
+        return response.read()
 
     def validate_frob(self, frob, perms):
         '''Lets the user validate the frob by launching a browser to
@@ -651,7 +648,7 @@ class FlickrAPI:
         auth_url = self.auth_url(perms, frob)
         webbrowser.open(auth_url, True, True)
         
-    def get_token_part_one(self, perms="read"):
+    def get_token_part_one(self, perms="read", auth_callback=None):
         """Get a token either from the cache, or make a new one from
         the frob.
         
@@ -660,7 +657,8 @@ class FlickrAPI:
         the method.
         
         If that fails (or if the token is no longer valid based on
-        flickr.auth.checkToken) a new frob is acquired.  The frob is
+        flickr.auth.checkToken) a new frob is acquired. If an auth_callback 
+        method has been specified it will be called. Otherwise the frob is
         validated by having the user log into flickr (with a browser).
         
         To get a proper token, follow these steps:
@@ -678,14 +676,39 @@ class FlickrAPI:
         run.
         
         perms
-            "read", "write", or "delete"           
-        
+            "read", "write", or "delete"
+        auth_callback
+            method to be called if authorization is needed. When not
+            passed, ``self.validate_frob(...)`` is called. You can
+            call this method yourself from the callback method too.
+
+            If authorization should be blocked, pass
+            ``auth_callback=False``.
+      
+            The auth_callback method should take ``(frob, perms)`` as
+            parameters.
+                                   
         An example::
         
             (token, frob) = flickr.get_token_part_one(perms='write')
             if not token: raw_input("Press ENTER after you authorized this program")
             flickr.get_token_part_two((token, frob))
+
+        Also take a look at ``authenticate_console(perms)``.
         """
+
+        # Check our auth_callback parameter for correctness before we
+        # do anything
+        authenticate = self.validate_frob
+        if auth_callback is not None:
+            if hasattr(auth_callback, '__call__'):
+                # use the provided callback function
+                authenticate = auth_callback
+            elif auth_callback is not False:
+                # Any non-callable non-False value is invalid
+                raise ValueError('Invalid value for auth_callback: %s'
+                        % auth_callback)
+
         
         # see if we have a saved token
         token = self.token_cache.token
@@ -711,12 +734,9 @@ class FlickrAPI:
             # get the frob
             LOG.debug("Getting frob for new token")
             rsp = self.auth_getFrob(auth_token=None, format='xmlnode')
-            self.test_failure(rsp)
 
             frob = rsp.frob[0].text
-
-            # validate online
-            self.validate_frob(frob, perms)
+            authenticate(frob, perms)
 
         return (token, frob)
         
@@ -740,7 +760,6 @@ class FlickrAPI:
         
         # get a token
         rsp = self.auth_getToken(frob=frob, auth_token=None, format='xmlnode')
-        self.test_failure(rsp)
 
         token = rsp.auth[0].token[0].text
         LOG.debug("get_token: new token '%s'" % token)
@@ -750,6 +769,96 @@ class FlickrAPI:
 
         return token
 
+    def authenticate_console(self, perms='read', auth_callback=None):
+        '''Performs the authentication, assuming a console program.
+
+        Gets the token, if needed starts the browser and waits for the user to
+        press ENTER before continuing.
+
+        See ``get_token_part_one(...)`` for an explanation of the
+        parameters.
+        '''
+
+        (token, frob) = self.get_token_part_one(perms, auth_callback)
+        if not token: raw_input("Press ENTER after you authorized this program")
+        self.get_token_part_two((token, frob))
+
+    @require_format('etree')
+    def __data_walker(self, method, **params):
+        '''Calls 'method' with page=0, page=1 etc. until the total
+        number of pages has been visited. Yields the photos
+        returned.
+        
+        Assumes that ``method(page=n, **params).findall('*/photos')``
+        results in a list of photos, and that the toplevel element of
+        the result contains a 'pages' attribute with the total number
+        of pages.
+        '''
+
+        page = 1
+        total = 1 # We don't know that yet, update when needed
+        while page <= total:
+            # Fetch a single page of photos
+            LOG.debug('Calling %s(page=%i of %i, %s)' %
+                    (method.func_name, page, total, params))
+            rsp = method(page=page, **params)
+
+            photoset = rsp.getchildren()[0]
+            total = int(photoset.get('pages'))
+
+            photos = rsp.findall('*/photo')
+
+            # Yield each photo
+            for photo in photos:
+                yield photo
+
+            # Ready to get the next page
+            page += 1
+
+    @require_format('etree')
+    def walk_set(self, photoset_id, per_page=50, **kwargs):
+        '''walk_set(self, photoset_id, per_page=50, ...) -> \
+                generator, yields each photo in a single set.
+
+        :Parameters:
+            photoset_id
+                the photoset ID
+            per_page
+                the number of photos that are fetched in one call to
+                Flickr.
+
+        Other arguments can be passed, as documented in the
+        flickr.photosets.getPhotos_ API call in the Flickr API
+        documentation, except for ``page`` because all pages will be
+        returned eventually.
+
+        .. _flickr.photosets.getPhotos:
+            http://www.flickr.com/services/api/flickr.photosets.getPhotos.html
+        
+        Uses the ElementTree format, incompatible with other formats.
+        '''
+
+        return self.__data_walker(self.photosets_getPhotos,
+                photoset_id=photoset_id, per_page=per_page, **kwargs)
+
+    @require_format('etree')
+    def walk(self, per_page=50, **kwargs):
+        '''walk(self, user_id=..., tags=..., ...) -> generator, \
+                yields each photo in a search query result
+
+        Accepts the same parameters as flickr.photos.search_ API call,
+        except for ``page`` because all pages will be returned
+        eventually.
+
+        .. _flickr.photos.search:
+            http://www.flickr.com/services/api/flickr.photos.search.html
+
+        Also see `walk_set`.
+        '''
+
+        return self.__data_walker(self.photos_search,
+                per_page=per_page, **kwargs)
+
 def set_log_level(level):
     '''Sets the log level of the logger used by the FlickrAPI module.
     
